"""Fixtures for testing.

This module provides custom fixtures for pytest that can be pluued into tests
across the entire test suite.
All fixtures defined under the fixtures package are auto plugged in automatically
by pyrig via the pytest_plugins mechanism.
"""
